// (C) Copyright 2013 Hewlett-Packard Development Company, L.P.
/**
 * @type (GroupAddView)
 * This is a view class for adding a new Group.
 */
define(['hpsum/presenter/group/GroupPresenter',
	'hpsum/presenter/node/NodePresenter',
    'hp/core/Router',
    'hp/core/UrlFragment',
    'hp/core/Validator',
    'hp/view/FormStateView',
    'hp/core/HelpMenu',
    'hp/core/Localizer',
    'jquery',
    'hp/lib/jquery.hpStackedPanels',
    'lib/jquery.selectBox',
    'lib/jquery.dataTables',
    'hp/lib/jquery.hpCollapsible',
    'hp/lib/jquery.hpSearchCombo',
    'hp/lib/jquery.hpToggle'],
function(presenter, nodePresenter, router, urlFragment, validator, FormStateView, helpMenu, localizer) { "use strict";
    	

	var AddView = ( function() {
			// consts can go here
			var ADD = '#hpsum-group-add',
			    CANCEL = '#hpsum-group-add-close',
			    RESET = "#hpsum-group-reset",
			    PANEL_SELECTOR = '#hpsum-group-add-panel-selector',
			    PANELS = '#hpsum-group-add-panels',
			    NAME = '#hpsum-group-name',
			    NAMEERR = '#hpsum-group-name-error',
			    DESC = '#hpsum-group-description',
			    ADD_NODES_TABLE = '#hpsum-group-add-nodes-table',
			    ROUTE_PREFIX = '/group/add',
			    BASELINES = '#hpsum-group-add-baselines-search',
            	HOTFIX = '#hpsum-group-add-hotfix-search',
            	SELECT = '#hpsum-group-select-all-nodes',
            	DESELECT = '#hpsum-group-deselect-all-nodes',
            	THROBBER = '#hpsum-group-add-throbber',
            	BASELINE_WRANING = '#hpsum-group-add-baseline-warning';

			/**
			 * @constructor
			 */
			function AddView() {

				var formStateView = new FormStateView(),
				    nodes = [],
				    addInProgress = false,
				    selectedAddNodes = [],
				    baselines='',
    				hotfixes='',
    				selectedBaselines = [];

				function changeLocation(location) {

					if (location.length > ROUTE_PREFIX.length) {
						$(PANELS).hpStackedPanels('setPanelByLocation', '#' + location);
					} else {
						$(PANELS).hpStackedPanels('setFirstPanel');
					}
				}

				function onItemAddSuccess(data) {
					$(ADD).attr('disabled', false);
					$(THROBBER).hide();
					if (data.hapi.hcode == 5032) {
						$(NAMEERR).text(localizer.getString('hpsum.group.groupAlreadyExists'));
						$(NAME).addClass("hp-error");
						$(NAMEERR).show();
						$(NAME).keyup(onNameEdit);
						addInProgress = false;
					} else {
						reset();
						addInProgress = false;
						$(CANCEL).trigger('click');

					}
				}

				function onNameEdit() {
					$(NAMEERR).hide();
					$(NAME).removeClass("hp-error");
					$(NAME).unbind('keyup', onNameEdit);
				}

				function onItemAddError(msg) {
					$(ADD).attr('disabled', false);
					$(THROBBER).hide();
					addInProgress = false;
					$(NAMEERR).text(msg);
					$(NAME).addClass("hp-error");
					$(NAMEERR).show();
				}

				function reset() {
					formStateView.reset();
					addInProgress = false;
					$(NAMEERR).hide();
					$(NAME).removeClass("hp-error");
				}

				function validate() {
					return validateName();
				}

				function validateName() {
					var success = true;
					var groupName = $(NAME).val();
					if ($(NAME).val() == '') {
						$(NAMEERR).text(localizer.getString('hpsum.node.required'));
						$(NAME).addClass("hp-error");
						$(NAMEERR).show();
						$(NAME).keyup(onNameEdit);
						success = false;
					} else if (!groupName.match(new RegExp(/^[a-z][a-z-0-9 ]*$/i))) {
						$(NAMEERR).text(localizer.getString('hpsum.group.inValidGroupName'));
						$(NAME).addClass("hp-error");
						$(NAMEERR).show();
						$(NAME).keyup(onNameEdit);
						success = false;
					} 
					
					return success;
				}

				function getSelectedNodes() {					
					return selectedAddNodes;
				}
			
				function getSelectedBaselines() {					
					selectedBaselines = [];
					if (($(BASELINES).val() != '' ) && ($(HOTFIX).val() != '')) {
						selectedBaselines = selectedBaselines.concat($(BASELINES).val());
						selectedBaselines = selectedBaselines.concat($(HOTFIX).val());
					} else if (($(BASELINES).val() == '') && ($(HOTFIX).val() != '')) {
						selectedBaselines = selectedBaselines.concat($(HOTFIX).val());
					} else if (($(BASELINES).val() != '') && ($(HOTFIX).val() == '')){
						selectedBaselines = selectedBaselines.concat($(BASELINES).val());
					}
				}
				
				function onAddItem() {
					if (validate()) {
						$(ADD).attr('disabled', true);
						$(THROBBER).show();
						if (!addInProgress) {
							addInProgress = true;
							var selectedNodes = getSelectedNodes();
							var trimmedGroupName = $.trim($(NAME).val());
							getSelectedBaselines();
							presenter.addItem({
								name : trimmedGroupName,
								description : $(DESC).val(),
								nodes : selectedNodes,
								baselines : selectedBaselines
							}, true, {
								success : onItemAddSuccess,
								error : onItemAddError
							});
						}
					}

				}

				function getNodesSuccess(results) {
					selectedAddNodes = [];
					nodes = results.hapi.output_data.nodes.node;
					for (var idx = 0; idx < nodes.length; ++idx) {
						nodePresenter.getNodeData({
							node_id : nodes[idx].node_id
						}, {
							success : function(data) {
								var nodeData = data.hapi,
									status = 'unknown',
									change = false,
									nodeStatus = nodeData.node_status,
									rebootRequired = "",
									nodeBaseline = localizer.getString('hpsum.group.noBaseline'),
									adtPackage = localizer.getString('hpsum.group.noAdtPackage');

								if (nodeData.node_state.match(/inventory|analysis|install$/i) ||
									nodeData.node_phase.match(/initialize/i)) {
									status = 'unknown';
									change = true;
								} else if (nodeData.node_state.match(/error/i) ||
										   nodeData.node_phase.match(/initialize|InActive/i)) {
									status = 'error';
								} else if (nodeData.node_state.match(/uan/i)) {
									if (nodeData.node_status.match(/No applicable/i)) {
										status = 'ok';
									} else {
										status = 'warning';
									}
								} else if (nodeData.node_state.match(/installdone/i)) {
									if (nodeData.return_code == -2 || nodeData.return_code == -3) {
										status = 'error';
										nodeStatus = localizer.getString('hpsum.node.installDoneWithErrors');
									} else {
										status = 'ok';
									}

									if (nodeData.return_code == 1) {
										rebootRequired = localizer.getString('hpsum.node.rebootRequired')
									}

								} else if (nodeData.node_state.match(/idle/i)) {
									if (nodeData.node_phase.match(/unknown/i)) {
										status = 'error';
									} else {
										status = 'warning';
									}
								}
								
								if (nodeData.baselines) {
									for (var i = 0; i < nodeData.baselines.length; i++) {
										var baseline = presenter.getBaselineDetailsforId(nodeData.baselines[i]);
										if (baseline) {
											if (baseline.name.match(/^Additional Package/i)) {
												adtPackage = baseline.name;
											} else {
												nodeBaseline = baseline.name;
											}
										}
									}
								}

								$(ADD_NODES_TABLE).dataTable().fnAddData({
									status : status,
									change : change,
									uri : nodeData.uri,
									node_id : nodeData.node_id,
									name : nodeData.name,
									type : nodeData.type,
									node_status : nodeStatus,
									state : nodeData.node_state,
									return_code : nodeData.return_code,
									reboot : rebootRequired,
									baseline : nodeBaseline,
									adtPackage : adtPackage
								});
							},
							error : function(errorMsg) {
							}
						}); 
					}
					
				}
				
				function onTableRow(e) {
						var aData = $(ADD_NODES_TABLE).dataTable().fnGetData(this),
							selectdNodesLenght = selectedAddNodes.length,
						    idx;
						if ($(this).hasClass('hp-selected')) {
							$(this).removeClass('hp-selected');
							for (idx = 0; idx < selectdNodesLenght; idx += 1) {
								if (selectedAddNodes[idx] === aData.node_id) {
									selectedAddNodes.splice(idx, 1);
									break;
								}
							}
							if (0 == selectedAddNodes.length) {
								$(DESELECT).attr('disabled', true);
							}
							else if (selectedAddNodes.length < nodes.length) {
								$(SELECT).attr('disabled', false);
							}
 						} else {
							$(this).addClass('hp-selected');
							selectedAddNodes = selectedAddNodes.concat(aData.node_id);
							$(DESELECT).attr('disabled', false);
							if (selectedAddNodes.length == nodes.length) {
								$(SELECT).attr('disabled', true);
							}
							else if (selectedAddNodes.length < nodes.length) {
								$(SELECT).attr('disabled', false);
							}
						}
				}
				
				function getNodesError(msg) {
				}

				function getBaseline(string, handlers) {
					var results = [];
					if (string && string.length > 0) {
						results = $.grep(baselines, function(option, index) {
							return (option.name.match(new RegExp(string, 'i')));
						});
					}
					handlers.success({
						count : results.length,
						members : results
					});
				}

				function getHotfixes(string, handlers) {
					var results = [];
					if (string && string.length > 0) {
						results = $.grep(hotfixes, function(option, index) {
							return (option.name.match(new RegExp(string, 'i')));
						});
					}
					handlers.success({
						count : results.length,
						members : results
					});
				}
          	
				function getBaselinesSuccess(searchResults) {
					baselines = $.grep(searchResults, function(option, index) {
						return !(option.name.match(new RegExp('^Additional', 'i')));
					});
					hotfixes = $.grep(searchResults, function(option, index) {
						return (option.name.match(new RegExp('^Additional', 'i')));
					});
				}

				function getBaselinesError(errorMsg) {

				}

				function onSelectAll() {
					$(ADD_NODES_TABLE + ' tbody tr').addClass('hp-selected');
					$(SELECT).attr('disabled', true);
					$(DESELECT).attr('disabled', false);
					selectedAddNodes = [];
					for (var idx = 0; idx < nodes.length; ++idx) {
						selectedAddNodes = selectedAddNodes.concat(nodes[idx].node_id);
					}
				}
				
				function onDeselectAll() {
					$(ADD_NODES_TABLE + ' tbody tr').removeClass('hp-selected');
					$(DESELECT).attr('disabled', true);			
					$(SELECT).attr('disabled', false);
					selectedAddNodes = [];
				}
				
				/**
				 * @public
				 * Intialize the group edit view.
				 */
				this.init = function() {

					$(PANELS).hpStackedPanels({
						panelSelector : PANEL_SELECTOR
					});

					/*$(CANCEL).click(function() {
						router.go(urlFragment.replaceView(router.location(), 'show'));
					});*/

					var dummyNodeData = [];

					$(ADD_NODES_TABLE).dataTable({
						oLanguage : {
						    sEmptyTable: localizer.getString("hpsum.group.noMoreNodesAvailable"),
						    sSearch: localizer.getString('hpsum.common.search')
						},
						bPaginate : false,
						sScrollY: "450px",
						bFilter : true,
						bInfo : false,
						bAutoWidth : false,
						bRetrieve : true,
						aaData : dummyNodeData,
						aoColumns : [
						{mDataProp: 'status',
							fnRender: function (oObj) {
								var statusCode;
								if (oObj.aData.change == true )
									statusCode =  '<div class="hp-status"><div class="hp-status-value hp-status-changing"></div></div>';
								else if(oObj.aData.status == "ok")
                                    statusCode =  '<div class="hp-status"><div class="hp-status-value hp-status-ok"><span class="hp-value">ok</span></div></div>';
                                else if(oObj.aData.status == "error")
                                	statusCode =  '<div class="hp-status"><div class="hp-status-value hp-status-error"><span class="hp-value">error</span></div></div>';
                                else if(oObj.aData.status == "warning")
                                	statusCode =  '<div class="hp-status"><div class="hp-status-value hp-status-warning"><span class="hp-value">warning</span></div></div>';
                                else if(oObj.aData.status == "unknown")
                                	statusCode =  '<div class="hp-status"><div class="hp-status-value hp-status-unknown"><span class="hp-value">unknown</span></div></div>';
                                return statusCode;
								/*
								 * Disable for now 
								 return '<div class="hp-switch hp-card"> <ol class="hp-up-ports">' +
								'<li class="hp-up-port">' + ' <div class="hp-condensed">' + statusCode +
								' </div>' + 
								'<div class="hp-full">' +
								statusCode + 
								'<div class="hp-form-contents"> ' +
								'<fieldset>' + '<ol> ' + ' <li> ' +
								'<div><a href="#/node' + '/show/r' + oObj.aData.uri + '/getData">' + oObj.aData.name + '</a><div></li>' +
								' <li> ' + '<div>' + oObj.aData.type + '</div>' + ' </li> ' +
								' <li> ' + '<div>' + oObj.aData.state + '</div>' + ' </li> ' +
								' <li> ' + '<div>' + oObj.aData.node_status + '</div>' + ' </li> ' +
								' <li> ' + '<div>' + oObj.aData.reboot + '</div>' + ' </li> ' +
								'</ol></fieldset></div></li></ol></div>';
								*/
							},
							sWidth: '10px', sClass: "hp-icon"
						},{
							mDataProp : 'name'
						}, {
							mDataProp : 'type'
						}, {
							mDataProp : 'node_status'
						}, {
							mDataProp : 'baseline'
						}, {
							mDataProp : 'adtPackage'
						}],
						aaSorting : [[1, "asc"]],
						bScrollCollapse : true
					});
					
					$(SELECT).live('click', function(ev) {
						onSelectAll.call(this);
						ev.preventDefault();
					});
					
					$(DESELECT).live('click', function(ev) {
						onDeselectAll.call(this);
						ev.preventDefault();
					});
					
					$(CANCEL).live('click', function(ev) {
						$(ADD_NODES_TABLE + " tbody").off('click', 'tr', onTableRow);
						router.go(urlFragment.replaceView(router.location(), 'show'));
						//ev.preventDefault();
					});
					
					$(ADD).live('click', function(ev) {
						onAddItem.call(this);
						ev.preventDefault();
					});

					$(RESET).live('click', function() {
						reset.call();
					});

					$(BASELINES).change(function (ev, value) {
						if (value == "" && $(HOTFIX).val() == "") {
							$(BASELINE_WRANING).hide();
						} else {
							$(BASELINE_WRANING).show();
						}
					});
					
					$(HOTFIX).change(function (ev, value) {
						if (value == "" && $(BASELINES).val() == "") {
							$(BASELINE_WRANING).hide();
						} else {
							$(BASELINE_WRANING).show();
						}
					});
					
					router.watch('group add section', ROUTE_PREFIX + '.*', {
						parent : ROUTE_PREFIX,
						change : changeLocation
					});

					$(BASELINES).hpSearchCombo({getResults:getBaseline});
                	$(HOTFIX).hpSearchCombo({getResults: getHotfixes});
                	$(BASELINE_WRANING).hide();
					this.resume();
				};

				/**
				 * @public
				 * Resume the group edit view.
				 */
				this.resume = function() {
					$(ADD_NODES_TABLE).dataTable().fnClearTable(true);
					$(ADD_NODES_TABLE + " tbody").on('click', 'tr', onTableRow);
					$(BASELINE_WRANING).hide();
					$(SELECT).attr('disabled', false);
					$(DESELECT).attr('disabled', true);
					$(THROBBER).hide();
					
					$(BASELINES).hpSearchCombo('set', {
						id : '',
						name : ''
					});
					$(HOTFIX).hpSearchCombo('set', {
						id : '',
						name : ''
					});
					$(RESET).trigger('click');
					$(NAMEERR).hide();
					$(NAME).removeClass("hp-error");
					nodePresenter.getAllNodes({
						success : getNodesSuccess,
						error : getNodesError
					});
					
					presenter.getAllBaselines({
           					success:getBaselinesSuccess,
           					error: getBaselinesError
           			});
				};

				/**
				 * @public
				 * Pause the group edit view.
				 */
				this.pause = function() {

				};
			}

			return AddView;
		}());

	return new AddView();

	});


